"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

 
import sys
from sdl2 import *
import sdl2.ext
import resources
from resources import R
from scene import Scene
from mainscene import MainScene
from menuscene import MenuScene
import random
import config
from config import C
from sdl2.sdlttf import TTF_Init, TTF_Quit
import sdl2.sdlmixer as sdlmixer
import audio
from level import Level


def init():
	config.init()
	sdl2.ext.init()
	TTF_Init()
	sdlmixer.Mix_Init(0)
	sdlmixer.Mix_OpenAudio(sdlmixer.MIX_DEFAULT_FREQUENCY, sdlmixer.MIX_DEFAULT_FORMAT, 2, 2048)
	audio.init()

def deinit():
	sdlmixer.Mix_CloseAudio()
	sdlmixer.Mix_Quit()
	TTF_Quit()

def run():
	random.seed(123)


	flags = 0
	if C.resizable: flags |= SDL_WINDOW_RESIZABLE

	window = sdl2.ext.Window(C.game_name, size=C.resolution, flags = flags)
	window.show()
	renderer = SDL_CreateRenderer(window.window, -1, SDL_RENDERER_ACCELERATED)
	SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255)

	SDL_RenderSetLogicalSize(renderer,C.resolution[0],C.resolution[1])

	resources.init(renderer)

	in_scene = Scene.MENU_SCENE

	levels = [
				(1,[11,14,17,18,20]),
				(2,[14,16,20,37,57]),
				(3,[13,18,25,99,111]),
				(4,[11,23])
				]

	lev_pair = []
	for dif, num in levels:
		for n in num:
			lev_pair.append((dif, n))

	levels = lev_pair

	is_random = False

	ALL_LEVELS_MODE = 1
	RANDOM_LEVEL_MODE = 2

	option = 0
	current_level = 0

	while True:
		current_scene = in_scene

		if in_scene == Scene.MAIN_SCENE:
			scene = MainScene(renderer, option == MenuScene.RANDOM_LEVEL)
			level = Level() #R.path("level003.lev", "levels"))
			if option == MenuScene.RANDOM_LEVEL:
				level.fullRandom()
				current_level = None
				scene.setLevel(level)
			else:
				level.automatic(levels[current_level][1], levels[current_level][0])
				scene.setLevel(level, current_level, len(levels))
		else:
			in_scene = Scene.MENU_SCENE
			scene = MenuScene(renderer)

		next_scene, params= scene.loop()

		if next_scene is False:
			return 0

		if next_scene is not True:
			in_scene = next_scene

		if current_scene == Scene.MENU_SCENE:
			option = params
			current_level = 0
		elif current_scene == Scene.MAIN_SCENE:
			if next_scene is True: # next level
				current_level += 1
				if current_level == len(levels):
					in_scene = Scene.MENU_SCENE


def main():
	init()
	r = run()
	deinit()
	sys.exit(r)

if __name__ == "__main__":
	main()
