"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import os
import os.path
import sdl2.ext
from sdl2 import *

class Animation:
	def __init__(self, directory, renderer, blend_mode=SDL_BLENDMODE_BLEND, tint=None):
		self._frames = []
		self._renderer = renderer

		files = [f for f in os.listdir(directory) if os.path.isfile(os.path.join(directory,f))]

		files.sort()


		for f in files:
			full_name = os.path.join(directory,f)
			surf = sdl2.ext.load_image(full_name)
			if  not surf:
				print"Couldn't load surface %s", full_name
				continue
			tex = SDL_CreateTextureFromSurface(renderer, surf)

			if blend_mode is not None:
				SDL_SetTextureBlendMode(tex, blend_mode)

			if tint is not None:
				SDL_SetTextureColorMod(tex, int(tint[0]*255.0), int(tint[1]*255.0), int(tint[2]*255.0))

			self._frames.append(tex)

		self._total_frames = len(self._frames)

		self._rect=  SDL_Rect(0,0,0,0)

		self._orig_w = surf.w
		self._orig_h = surf.h


	@property
	def width(self):
		return self._orig_w


	@property
	def height(self):
		return self._orig_h


	@property
	def renderer(self):
		return self._renderer


	@property
	def frames(self):
		return self._total_frames


	def paint(self, frame, rect, tint=None):
		texture = self._frames[frame]
		if tint is not None:
			SDL_SetTextureColorMod(texture, tint[0],tint[1],tint[2])
		SDL_RenderCopy(self._renderer, texture, None, rect)
