"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from resources import R
from sdl2 import *
import random
from clickable import Clickable
from entity import Entity
import numpy as N

class Cell(Entity):
	def __init__(self, scene, pos, scale=1, listener=None):
		Entity.__init__(self, scene)
		tex_name = "cell.png"
		self._texture1 = R.texture(tex_name,SDL_BLENDMODE_ADD, N.array((1,0.2,0.5))*0.8)
		self._texture2 = R.texture(tex_name,SDL_BLENDMODE_ADD, N.array((1*0.2,0.2,0.7))*0.8)
		self._texture3 = R.texture(tex_name,SDL_BLENDMODE_ADD, N.array((1*0.2,0.7,0.5*0.2))*0.8)
		self._pos = pos
		self._scale = scale * 100
		self._rect = SDL_Rect(0,0,1,1)
		self.scale(scale)
		self.moveTo(pos)
		self._renderer = self.scene.renderer
		self._t_offset = random.random()*10000.0
		self._evt_handler = Clickable(self, listener or self)
		self._t = 0


	def entityType(self):
		return Entity.CELL_TYPE

	def categories(self):
		from scene import Scene
		return (Scene.RENDERABLE, Scene.LISTENER)

	def scale(self, scale):
		self._scale = scale * 100.0
		self._scale2 = self._scale * 0.5
		self._rect.x = int(self._pos[0] - self._scale2+0.5)
		self._rect.y = int(self._pos[1] - self._scale2+0.5)
		self._rect.w = int(self._scale+0.5)
		self._rect.h = self._rect.w


	def moveTo(self, pos):
		self._pos = pos
		self._rect.x = int(pos[0] - self._scale2+0.5)
		self._rect.y = int(pos[1] - self._scale2+0.5)


	@property
	def position(self):
		return self._pos

	def processEvent(self, evt):
		return self._evt_handler(evt)

	def isPointOver(self, x, y):
		dx = x - self._pos[0]
		dy = y - self._pos[1]

		# bounding box check
		if dx > self._scale2 or dx < -self._scale2 or dy > self._scale2 or dy < -self._scale2:
			return False

		# disc check
		if dx*dx+dy*dy > self._scale2*self._scale2:
			return False

		return True

	def update(self, t, dt):
		self._t = t
		return None

	def paint(self):
		t = self._t + self._t_offset
		rect = self._rect #SDL_Rect(self._rect.x, self._rect.y, self._rect.w, self._rect.h)
		renderer = self._renderer
		SDL_RenderCopyEx(renderer, self._texture1.texture, None, rect, 0.01*t, None, SDL_FLIP_NONE)
		SDL_RenderCopyEx(renderer, self._texture2.texture, None, rect, -0.0137*t, None, SDL_FLIP_NONE)
		SDL_RenderCopyEx(renderer, self._texture3.texture, None, rect, -0.00732*t, None, SDL_FLIP_NONE)

	def _onClick(self,who,x,y):
		print "Cell Clicked!"

	def __str__(self):
		return "[Cell %s]"%self._entity_id
