"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from sdl2 import *

class Clickable:
	def __init__(self, clickable, listener=None):
		if listener is None:
			listener = clickable
		self._clickable = clickable
		self._check_over_fn = getattr(clickable, 'isPointOver')
		self._hover = False
		self._pressed = False
		self._on_press = getattr(listener,"_onPress",None)
		self._on_release = getattr(listener,"_onRelease",None)
		self._on_click = getattr(listener,"_onClick",None)
		self._on_drag = getattr(listener,"_onDrag",None)
		self._on_move = getattr(listener,"_onMove",None)
		self._on_hover = getattr(listener,"_onHover",None)
		self._on_blur = getattr(listener,"_onBlur",None)


	def _dispatch(self, cb, *args, **kargs):
		if cb is not None:
			cb(self._clickable, *args, **kargs)


	def __call__(self, event):
		consumed = False
		event_type = event.type
		if event_type == SDL_MOUSEMOTION:
			if self._pressed:
				self._dispatch(self._on_drag, x=event.motion.x, y=event.motion.y)
			if self._check_over_fn(event.motion.x, event.motion.y):
				if not self._pressed:
					self._dispatch(self._on_move, x=event.motion.x, y=event.motion.y)
				if not self._hover:
					self._hover = True
					self._dispatch(self._on_hover)
			else:
				if self._hover:
					self._hover = False
					self._dispatch(self._on_blur)

		elif event_type == SDL_MOUSEBUTTONDOWN:
			if not self._check_over_fn(event.button.x, event.button.y):
				return False
			if event.button.button != SDL_BUTTON_LEFT:
				return False
			self._hover = True
			self._pressed = True
			self._dispatch(self._on_press, x=event.button.x, y=event.button.y)
			consumed = True
		elif event_type == SDL_MOUSEBUTTONUP:
			if not self._pressed or not event.button.button == SDL_BUTTON_LEFT:
				return False
			self._pressed = False
			self._dispatch(self._on_release, x=event.button.x, y=event.button.y)
			if self._check_over_fn(event.button.x, event.button.y):
				self._dispatch(self._on_click, x=event.button.x, y=event.button.y)

		return consumed

	@property
	def hover(self):
		return self._hover

	@property
	def pressed(self):
		return self._pressed



