"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

class Entity:

	CELL_TYPE=1
	SPLINE_TRAIL_TYPE=2
	STATIC_SPRITE_TYPE = 3
	SPRITE_TYPE=4
	ANIM_SPRITE_TYPE=5
	PARTICLES_TYPE=5
	TEXT_TYPE=6
	CONTAINER_TYPE = 7
	TIMER_TYPE = 8
	BUTTON_TYPE = 8


	def __init__(self, scene, z=0):
		self._added_to = []
		self._entity_id = 0
		self._scene = None
		self._destroyed = False
		self._z_order = z

		if scene: self.register(scene)


	def entityType(self): #override this
		raise Exception("Override this")


	def categories(self): # override this
		raise Exception("Override this")

	def register(self, scene):
		if self._scene:
			self.unregister()

		self._scene = scene
		self._entity_id = scene.getNewEntityId()
		scene._registerEntity(self)

	def unregister(self):
		if self._scene:
			self._scene._unregisterEntity(self)
		self._scene = None

	def destroy(self, *args, **kargs): # take any arguments so it can be used as callback
		print "Destroying ",self.__class__
		self.unregister()
		self._destroyed = True

	@property
	def scene(self):
		return self._scene

	@property
	def entity_id(self):
		return self._entity_id

	@property
	def destroyed(self):
		return self._destroyed

	@property
	def z_order(self):
		return self._z_order
