"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


class PackedArrayItem:
	def __init__(self, array, id, loc, getters):
		self._id = id
		self._loc = loc
		self._valid = True
		self._getters = getters
		self._array = array

		self.on_signal = None

	def value(self):
		return self._getters[0](self._loc)

	def params(self):
		return self._getters[1](self._loc)

	@property
	def array(self):
		return self._array

	@property
	def id(self):
		return self._id

	@property
	def loc(self):
		return self._loc


class PackedArray:
	def __init__(self, initial_size):
		self._size = initial_size
		self._tot_items = 0
		self._id2loc= {}
		self._loc2id = []
		self._items = {} # indexed by id
		self._max_id = 0

	def _resize(self, new_size):
		pass

	def _moveData(self, to_loc, from_loc):
		pass

	def _getGetters(self):
		pass

	def _add(self):
		if self._size == self._tot_items:
			self._size *= 2
			self._resize(self._size)

		loc = self._tot_items
		id = self._max_id

		self._tot_items += 1
		self._max_id += 1

		self._loc2id.append(id)
		self._id2loc[id] = loc

		return loc,id


	def _addMany(self, tot):
		if self._size < self._tot_items + tot:
			self._size = 2*self._size + tot
			self._resize(self._size)

		initial_id = self._max_id
		initial_loc = self._tot_items

		self._loc2id.extend(xrange(initial_loc, initial_loc+tot))

		for i in xrange(tot):
			self._id2loc[initial_id+i] = initial_loc+i

		self._tot_items += tot
		self._max_id += tot

		return initial_loc, initial_id


	def _remove(self, id):
		loc_to = self._id2loc[id]
		loc_from = self._tot_items-1

		self._moveData(loc_to, loc_from)

		id_from = self._loc2id[loc_from]
		self._id2loc[id_from] = loc_to
		self._loc2id[loc_to] = id_from

		if id in self._items:
			self._items[id]._valid = False
			del self._items[id]

		if id_from in self._items:
			self._items[id_from]._loc = loc_to

		del self._id2loc[id]
		self._loc2id.pop()
		self._tot_items -= 1

		return self._tot_items


	def loc(self, id):
		return self._id2loc[id]


	@property
	def size(self):
		return self._tot_items


	def getProxy(self, id, on_signal = None):
		try:
			return self._items[id]
		except:
			item = PackedArrayItem(self, id, self.loc(id), self._getGetters())
			self._items[id] = item
			item.on_signal = on_signal

			return item

	def signalProxies(self):
		for item in self._items.values():
			if item.on_signal: item.on_signal()

	def __del__(self):
		for item in self._items.values():
			item._valid = False
