"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


USER = 1
ENEMY = 2

from resources import R


class Player:
	def __init__(self, scene, type, blob, current_cell, power_dial, turn_timer):
		self._scene = scene
		self.type = type
		self.current_cell = current_cell
		self.is_animating = False
		self.blob = blob
		self.power = power_dial
		self.turn_timer = turn_timer
		self._kill_mode = False

	def __str__(self):
		if self.type == USER: return "User"
		elif self.type == ENEMY: return "Enemy"
		return "???"

	def remainingTime(self):
		return self.turn_timer.remainingTime()

	def otherPlayer(self):
		if self.type == USER: return self._scene.getPlayer(ENEMY)
		elif self.type == ENEMY: return self._scene.getPlayer(USER)

	def setKillMode(self, kill_mode):
		self._kill_mode = kill_mode

	def isLocked(self, min_power_level=None):
		a = self.is_animating
		b = len(self._scene.reachableCells(self.current_cell))
		c = self.canJump(min_power_level)
		lock = (not a and b==0 and not c)
		print "LOCKED? ",self, a, b, c, ("LOCKED" if lock else "")
		return lock


	def canJump(self, min_power_level = None):
		if min_power_level is None:
			if self.type == ENEMY:
				min_power_level = 3
			elif self.type == USER:
				min_power_level = 2

		return self.power.getLevel() >= min_power_level

	def canPlayNextTurn(self):
		return not self.is_animating and self.turn_timer.isFull()

	# make a movement
	def play(self, on_done = None, cb_data = None):
		if not self.canPlayNextTurn():
			return

		s = self._scene
		other = self.otherPlayer()

		if self._kill_mode:
			print "KILL MODE >>>>> E: %s, U:%s"%(self.current_cell, other.current_cell)
			# follow the longest route if we'll eventually hit the user or
			# can jump to it
			longest_route = self.getLongestRoute(self.current_cell, True)
			print "LR: ",map(str,longest_route)
			if other.current_cell in longest_route:
				self.moveTo(longest_route[1]) #
				print "A", map(str,longest_route)
				return True

			lr_len = len(longest_route)-1 # we ignore the first one which is the one we're in

			# no longest route, but we can jump to the player now or eventually
			if lr_len + self.power.getLevel() >= 3:
				if lr_len <= 0:
					print "B", map(str, longest_route)
					self.jumpTo(other.current_cell)
				else:
					print "C", map(str, longest_route)
					self.moveTo(longest_route[1])
				return True

			shortest_route = self.getShortestRoute(self.current_cell, other.current_cell, False)
			print "D", str(shortest_route)
			if shortest_route:
				if shortest_route == self.current_cell:
					self.moveTo(other.current_cell)
				else:
					self.moveTo(shortest_route)
				return True
			else:
				return False
		else:
			# maybe chase user

			if s.getLink(self.current_cell, other.current_cell):
				self.moveTo(other.current_cell)
				return True

			# if there's only one route, take it
			targets = s.reachableCells(self.current_cell)
			if len(targets) == 1:
				self.moveTo(targets[0])
				return True

			# otherwise, see the route where we can make the most harm Muahahahah!
			best = self.getLongestRoute(self.current_cell, False)
			if best is not None:
				self.moveTo(best)
				return True

			# otherwise, see if we can jump...
			if self.power.getLevel() == 3:
				n = self.getLongestRoute(None, False)
				if n is not None:
					self.jumpTo(n)
					return True

		return False

	def quickModeTimer(self):
		self.turn_timer.fullSpeed()


	# if full_route: get all the nodes in the route from_node->to_node
	# including endpoints
	# otherwise, just return the next node to take when going from_node->to_node
	def getShortestRoute(self, from_node, to_node, full_route=False):
		next_node = {}
		to_visit = {from_node:0}
		frozen = set()
		s = self._scene

		while to_visit: # a vanilla Dijkstra
			current,distance= min(to_visit.iteritems(), key=lambda k_v: k_v[1])

			for neighbor in s.reachableCells(current):
				if neighbor in frozen: continue
				new_dist = distance + 1
				if neighbor not in to_visit or to_visit[neighbor] > new_dist:
					to_visit[neighbor] = new_dist
					next_node[neighbor] = (current,new_dist)
			frozen.add(current)
			del to_visit[current]
			if current == to_node: break

		if not to_node in next_node:
			return [] if full_route else None

		if full_route:
			route = [to_node]
			while to_node in next_node:
				next_node = next_node[to_node][0]
				route.append(next_node)
				to_node = next_node

			route.reverse()
			return route
		else:
			for k,v in next_node.iteritems():
				print ">> %s, %s"%(k,v[0])
			return next_node[to_node][0]


	# if full_route: get all the nodes in the longest route starting at from_node
	# including endpoints
	# otherwise, just return the next node to take when taking the longest route
	# if from_node is None, return the node with the largest route
	def getLongestRoute(self, from_node, full_route=False):
		next_node = {}
		visited = set()
		s = self._scene

		def calc(node): # TODO: change this to iterative
			if node in visited:
				return 0
			visited.add(node)
			if node in next_node:
				return next_node[node]
			best_n = None
			best_d = 0
			for adj_nodes in s.reachableCells(node):
				dist =calc(adj_nodes)
				if dist+1 > best_d:
					best_d = dist+1
					best_n = adj_nodes
			next_node[node] = (best_n, best_d)
			return best_d

		if from_node is None:
			for n in s.allCells():
				calc(n)
			#see which node has the longest route
			if not full_route:
				if len(next_node):
					node,_ = max(next_node.iteritems(), key=lambda nd:nd[1][1])
					return node
				else:
					return None

		else:
			calc(from_node)

		if not len(next_node):
			return [] if full_route else None

		if full_route:
			route = []
			seen = set()
			while from_node:
				if from_node in seen:
					break

				seen.add(from_node)
				route.append(from_node)

				if from_node in next_node:
					from_node = next_node[from_node][0]
				else:
					break

			return route

		else:
			return next_node[from_node][0]



	def moveTo(self, cell):
		self._scene.moveLinks(self.current_cell, cell, 0, self.type)
		self.turn_timer.reset()

	def jumpTo(self, cell):
		self._scene.moveLinks(self.current_cell, cell, 3, self.type)
		self.turn_timer.reset()
		self.power.clearLevel()





