"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from entity import Entity
from resources import R
from particles import Particles
from sdl2 import *
import random
import numpy as N

class PowerDial(Entity):
	def __init__(self, scene, pos):
		Entity.__init__(self, scene)
		self._max_level = 3
		self._sprites = []
		self._level = 0
		self._is_active = False

		blend_mode = SDL_BLENDMODE_ADD

		self._textures = [
					R.texture("bokeh5b.png", blend_mode = blend_mode, tint = None),
					R.texture("bokeh5.png", blend_mode = blend_mode, tint = None),
					R.texture("bokeh5.png", blend_mode = blend_mode, tint = None),
					R.texture("bokeh5.png", blend_mode = blend_mode, tint = None),
		        ]

		for i in xrange(len(self._textures),self._max_level):
			self._textures.append(self._textures[-1])

		self._particles_texture = R.texture("sparkle.png", blend_mode = SDL_BLENDMODE_ADD, tint = None)

		self._particles = None

		for i in xrange(self._max_level):
			s = scene.newStaticSprite((i*60+pos[0],pos[1]), self._textures[0], scale = 0.5,z=2)
			self._sprites.append(s)


	def _particleGenerator(self, pos):
		while True:
			dx = 2.*random.random()-1.
			dy = 2.*random.random()-1.

			r = dx*dx+dy*dy

			if r < 1.0 and r>0.5:
				break

		speed =N.array([dx,dy])
		r = 15
		return pos+speed*(r-2), pos+speed*r

	@property
	def width(self):
		return 60*(self._max_level-1)

	@property
	def height(self):
		return self._sprites[0].height


	def destroy(self):
		for s in self._sprites:
			s.destroy(self)
		Entity.destroy(self)

	def entityType(self):
		return Entity.CONTAINER_TYPE


	def categories(self):
		from scene import Scene
		return (Scene.CONTAINER,Scene.STATIC)


	def incLevel(self):
		self.deactivate()
		if self._level == self._max_level:
			return

		s =  self._sprites[self._level]
		self._level += 1
		s.changeTexture(self._textures[self._level])


	def clearLevel(self):
		self.deactivate()
		for s in self._sprites:
			s.changeTexture(self._textures[0])

		self._level = 0


	def getLevel(self):
		return self._level


	def canActivate(self):
		return self._level > 1

	def activate(self):
		if self._level == 0:
			return
		if self._particles:
			self._particles.destroy()

		self._particles = Particles(
					self._scene, self._particles_texture,
					pos = self._sprites[self._level-1].position,
					scale = 0.5,
					max_particles = 20,
					max_particle_life = 100,
					creation_rate = 100,
					custom_generator = self._particleGenerator,
					z = 5
					)

		self._is_active = True

	def isActive(self):
		return self._is_active

	def deactivate(self):
		if not self._is_active:
			return

		self._particles.shutdown()
		self._particles = None
		self._is_active = False







