"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


class Rect:
	__slots__= ('x','y','w','h')

	def __init__(self, x, y, w=None, h=None, x2=None, y2=None):
		self.x = x
		self.y = y

		if x2 is not None:
			if w is not None:
				raise TypeError("Can't specify both x2 and w when creating a Rect")
			self.x2 = x2
		else:
			if w is None:
				self.w = 0
			else:
				self.w = w

		if y2 is not None:
			if h is not None:
				raise TypeError("Can't specify both y2 and h when creating a Rect")
			self.y2 = y2
		else:
			if h is None:
				self.h = 0
			else:
				self.h = h


	@property
	def x2(self):
		return self.x + self.w


	@x2.setter
	def x2(self, x):
		self.w = x - self.x


	@property
	def y2(self):
		return self.y + self.h


	@y2.setter
	def y2(self, y):
		self.h = y -self.y


	def clone(self):
		return self.__class__(self.x, self.y, self.w, self.h)


	def contains(self, pt):
		if pt[0] < self.x: return False
		if pt[1] < self.y: return False
		if pt[0] > self.x2: return False
		if pt[1] > self.y2: return False
		return True

	# other can be a point or a rect
	def extend(self, other):
		if other is None:
			return

		x2 = self.x2
		y2 = self.y2

		try:
			self.x = min(self.x, other.x)
			self.y = min(self.y, other.y)
			self.w = max(x2,other.x2)-self.x
			self.h = max(y2,other.y2)-self.y
		except:
			self.x = min(self.x, other[0])
			self.y = min(self.y, other[1])
			self.w = max(self.x2, other[0])-self.x
			self.h = max(self.y2, other[1])-self.y

