"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from sdl2 import *
import sdl2.ext
import sys
import weakref
from texture import Texture
from animation import Animation
from text import Font
import os.path
from audio import Sound, Music



def cached(group = None, weak=True):

	def deco(func):
		def memo(self, *args, **kwargs):
			if group is not None:
				k = (group, args, tuple(kwargs.items()))

				try:
					hash(k)
				except TypeError:
					k = '%s:%s' % (group, repr(k))

				cache = self._weak_cache if weak else self._strong_cache

				try:
					return cache[k]
				except KeyError:
					result = func(self, *args, **kwargs)
					cache[k] = result
					return result
			else:
				return func(self, *args, **kwargs)

		return memo

	return deco


class Resources:
	def __init__(self, renderer):
		self._resources= os.path.join(os.path.dirname(__file__),  "resources")
		self._weak_cache = weakref.WeakValueDictionary()
		self._strong_cache = {}
		self._renderer = renderer



	@property
	def renderer(self):
		return self._renderer

	def setRenderer(self, renderer):
		print "Set resources renderer: ",renderer
		self._renderer = renderer

	def path(self, resource_file, subdir=None):
		if subdir:
			path = os.path.join(self._resources,subdir)
		else:
			path = self._resources
		return os.path.join(path,resource_file)


	@cached("SURFACE")
	def surface(self, resource_file):
		p = self.path(resource_file,"images")
		try:
			s = sdl2.ext.load_image(p)
		except:
			print "Error loading image: ",p
			print "Supported types:"
			print " ".join(sdl2.ext.get_image_formats())
			raise
		if not s:
			print"Couldn't load surface %s", p
		return s

	@cached("TEXTURE")
	def texture(self, resource_file, blend_mode=SDL_BLENDMODE_BLEND, tint=None):
		return Texture(self.surface(resource_file), self._renderer, blend_mode, tint)

	@cached("ANIMATION")
	def animation(self, resource_name, blend_mode=SDL_BLENDMODE_BLEND, tint=None):
		return Animation(self.path(resource_name,"animations"), self._renderer, blend_mode, tint)

	@cached("FONT")
	def font(self, resource_name, size=30):
		return Font(self.path(resource_name, "fonts"), self._renderer, size)

	@cached("AUDIO")
	def music(self, resource_name):
		return Music(self.path(resource_name, "audio"))

	@cached("AUDIO")
	def sound(self, resource_name):
		return Sound(self.path(resource_name, "audio"))

def init(renderer):
	global R
	print "Initializing resources"
	R.setRenderer(renderer)




R = Resources(None)
