"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from entity import Entity
from config import C

class Score(Entity):
	def __init__(self, scene, pos, initial_value=0):
		Entity.__init__(self, scene)
		self._pos = pos
		self._initial_value = initial_value
		self._target_value = initial_value
		self._current_value = self._target_value
		self._shown = self._current_value
		self._text = scene.newText(C.mono_font,pos,"%s"%initial_value, center_x = True, size=50)


	def destroy(self):
		self._text.destroy()
		Entity.destroy(self)


	def entityType(self):
		return Entity.CONTAINER_TYPE


	def categories(self):
		from scene import Scene
		return (Scene.CONTAINER,)

	@property
	def value(self):
		return self._target_value


	def isZero(self):
		return self._target_value <= 0

	def update(self, t, dt):
		s = self._shown
		self._current_value += (self._target_value - self._current_value) * (dt / 100.0)
		self._shown = max(0, int(self._current_value + 0.5))
		if self._shown != s:
			self._text.setText("%s"%self._shown)


	def setValue(self, val):
		self._target_value = val


	def delta(self, incr):
		self.setValue(max(0,self._target_value + incr))

	def reset(self, value):
		self._current_value = value
		self._target_value = value


