"""
The MIT License (MIT)

Copyright (c) 2017 Guillermo Romero Franco (AKA Gato)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from entity import Entity
from resources import R
from sdl2 import *

class TurnGauge(Entity):
	def __init__(self, scene, pos, time=2, on_full=None, cb_data = None, scale=1, tint = None):# time in seconds for the bar to fill
		Entity.__init__(self, scene)

		#blend_mode = SDL_BLENDMODE_ADD
		blend_mode = SDL_BLENDMODE_BLEND

		self._gauge_pos = pos
		self._time = time * 1000.0 # time in milliseconds
		self._on_full = on_full
		self._cb_data = cb_data
		self._elapsed = 0


		g_tex= R.texture("turn_gauge.png", blend_mode = blend_mode, tint = tint)
		self._gauge_sprite = scene.newStaticSprite(pos, g_tex, z=2, centered = False, scale=scale)

		b_tex= R.texture("turn_gauge_bar.png", blend_mode = blend_mode, tint = tint)

		bar_pos = (pos[0] + (g_tex.width - b_tex.width)*0.5*scale,
		           pos[1] + (g_tex.height - b_tex.height)*0.5*scale)

		self._bar_sprite = scene.newStaticSprite(bar_pos, b_tex, z=2, centered = False, scale = scale)

		self._scale = scale
		self._full = False

	@property
	def width(self):
		return self._gauge_sprite.width

	@property
	def height(self):
		return self._gauge_sprite.height

	def destroy(self):
		self._gauge_sprite.destroy()
		self._bar_sprite.destroy()
		Entity.destroy(self)

	def entityType(self):
		return Entity.CONTAINER_TYPE


	def categories(self):
		from scene import Scene
		return (Scene.CONTAINER,)

	def update(self,t, dt):
		if self._full:
			return None

		do_signal = None
		self._elapsed += dt
		t = self._elapsed / self._time
		if t >= 1.0:
			t = 1.0
			self._elapsed = self._time
			self._full = True
			if self._on_full:
				do_signal = lambda:self._on_full(self, self._cb_data)

		self._bar_sprite.scale(t*self._scale, self._scale)

		return do_signal

	def remainingTime(self):
		return (self._time - self._elapsed)*0.001

	def fullSpeed(self):
		self._time = 1

	def reset(self):
		self._full = False
		self._elapsed = 0


	def isFull(self):
		return self._full or self._time == 1


